#include<stdio.h>
#include<stdlib.h>
#include<string.h>

#include "types.h"
#include "defaults.h"
#include "warning.h"
#include "mystring.h"
#include "X.h"
#include "pixmap.h"
#include "loop.h"

//: prvpix

kq_pixmap *prvpix = NULL;

//: rescalePixmap

void rescalePixmap (XImage *ximage, Pixmap *pxm)
/*********************************************/
{
  XImage *ipom;
  GC lgc;
  int i,j,x,x2,y;
  unsigned long pixel;

  if(*pxm) XFreePixmap(display, *pxm);  // Neni-li pxm None, odallokuji to
  if(ratio == 0 || !ximage){
    *pxm = None;
    return;
  }
  cekalose = 1;

  // Pripravim novou pixmapu:

  *pxm = XCreatePixmap(display, root,
		       ximage->width*ratio, ximage->height*ratio, ximage->depth);
  lgc = XCreateGC(display, *pxm, 0, NULL);

  if(ratio <= IMAGRAT){
    if(ratio == 1){  // Pomer je 1:1 -> neni treba aplikovat scale:
      XPutImage(display, *pxm, lgc, ximage, 0, 0, 0, 0,
		ximage->width, ximage->height);
      XFreeGC(display, lgc);
      return;
    }
    ipom = XCreateImage (display,
			 CopyFromParent, ximage->depth,
			 ximage->format, ximage->xoffset, 
			 mymalloc(ximage->bytes_per_line*ratio*
				ximage->height*ratio),
			 ximage->width*ratio, ximage->height*ratio,
			 ximage->bitmap_pad, ximage->bytes_per_line*ratio);
  }

  for(y=0; y<ximage->height; y++)
    for(x=0; x<ximage->width; x++){
      pixel = XGetPixel(ximage, x, y);    // Pro kazdy pixel ze zdrojoveho obrazku
      if(ratio <= IMAGRAT)
	for(j=y*ratio; j<(y+1)*ratio; j++)
	  for(i=x*ratio; i<(x+1)*ratio; i++)
	    XPutPixel(ipom, i, j, pixel); // vykreslim ctverec do pomocneho ximage
      else{
	// nebo vykreslim cely radek do pixmapy:

	for(x2 = x+1; x2<ximage->width && XGetPixel(ximage, x2, y) == pixel; x2++);
	XSetForeground(display, lgc, pixel);
	XFillRectangle(display, *pxm, lgc,
		       x*ratio, y*ratio, ratio*(x2-x), ratio);
	x = x2-1;
      }
    }

  if(ratio <= IMAGRAT)
    XPutImage(display, *pxm, lgc, ipom, 0, 0, 0, 0,
	      ipom->width, ipom->height);

  XFreeGC(display, lgc);
}

//: nactiXPM

Pixmap nactiXPM (char *filename, XImage **xim, Pixmap *shape, XImage **xish)
/*************************************************************************/
{
  Pixmap vysledek;
  XImage *ximage;
  XImage *shapeimage;
  char *fullname;

  // Prida ke jmenu cestu k obrazku:
  fullname = joinstrings(DATADIR, IMGDIR, filename, IMGSUFFIX,  NULL);

  // Nactu image pomoci X-ove funkce:
  if(XpmReadFileToImage(display, fullname, &ximage, &shapeimage, NULL))
    error("I can't open \"%s\"", fullname);
  free(fullname);

  vysledek = None;  // dosadim None, abych dal najevo, ze onu pixmapu nechci smazat
  rescalePixmap(ximage, &vysledek);  // Vytvorim pixmapu barev.

  if(xim) *xim = ximage;       // XImage barev vratim
  else XDestroyImage(ximage); // nebo odallokuji.

  if(shape){
   *shape = None;
   rescalePixmap(shapeimage, shape);  // Vytvorim pixmapu alfa kanalu
  }
  if(xish) *xish = shapeimage;     // XImage alfa kanalu vratim
  else if(shapeimage) XDestroyImage(shapeimage); // nebo odalokuji

  return vysledek;
}

//: nactikqXPM

kq_pixmap *nactikqXPM (char *filename, char lokalni)
/******************************************************/
{
  kq_pixmap *vysledek;

  vysledek = (kq_pixmap*)mymalloc(sizeof(kq_pixmap));
  vysledek->barvy = nactiXPM(filename, &vysledek->ibarvy,
			     &vysledek->akanal, &vysledek->ikanal);
  vysledek->scaled = ratio;
  vysledek->lokalni = lokalni;

  vysledek->nazev = dupstr(filename);

  vysledek->next = prvpix;
  prvpix = vysledek;

  return vysledek;
}

//: freekqPixmap

void freekqPixmap (kq_pixmap *pxm)
{
  kq_pixmap **pom;

  for(pom = &prvpix; *pom && *pom != pxm; pom = &(*pom)->next);
  if(!pom){
    warning("pixmapa nenalezena v seznamu");
    return;
  }

  *pom = pxm->next;
  free(pxm->nazev);
  if(pxm->barvy) XFreePixmap(display, pxm->barvy);
  if(pxm->akanal) XFreePixmap(display, pxm->akanal);
  XDestroyImage(pxm->ibarvy);
  XDestroyImage(pxm->ikanal);
  free(pxm);
}

//: deletelpixmaps

void deletelpixmaps ()
{
  kq_pixmap *oriprv, *pom;

  pom = oriprv = NULL;

  while(prvpix){
    if(prvpix->lokalni){
      freekqPixmap(prvpix);
      if(pom) pom->next = prvpix;
    }
    else{
      pom = prvpix;
      if(!oriprv) oriprv = prvpix;
      prvpix = prvpix->next;
    }
  }
  prvpix = oriprv;
}

//: deleteApixmaps

void deleteApixmaps ()
{
  kq_pixmap *pom;

  while(prvpix){
    pom = prvpix->next;
    free(prvpix->nazev);
    if(prvpix->barvy) XFreePixmap(display, prvpix->barvy);
    if(prvpix->akanal) XFreePixmap(display, prvpix->akanal);
    XDestroyImage(prvpix->ibarvy);
    XDestroyImage(prvpix->ikanal);
    free(prvpix);
    prvpix = pom;
  }
}

//: fixpix

kq_pixmap *fixed_pixmaps = NULL;
int pocet_fixpix = 0;

void fix_pix ()
{
  kq_pixmap *pom;

  if(pocet_fixpix) warning("funkce fix_pix() je spustitelna jen jednou");

  for(pom = prvpix; pom; pom = pom->next) pom->Cislo = --pocet_fixpix;
  fixed_pixmaps = prvpix;
  prvpix = NULL;
}
